@@ -1,5 +1,4 @@
 /* Internal bookmarks support */
-/* $Id: bookmarks.c,v 1.150.2.8 2005/04/06 08:52:24 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -16,18 +15,18 @@
 #include "bookmarks/backend/common.h"
 #include "bookmarks/bookmarks.h"
 #include "bookmarks/dialogs.h"
+#include "config/home.h"
 #include "config/options.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/home.h"
-#include "modules/module.h"
+#include "main/module.h"
+#include "main/object.h"
 #include "protocol/uri.h"
-#include "sched/task.h"
+#include "session/task.h"
 #include "terminal/tab.h"
 #include "util/conv.h"
 #include "util/hash.h"
 #include "util/lists.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/secsave.h"
 #include "util/string.h"
 
@@ -85,10 +84,10 @@ static enum evhook_status bookmark_change_hook(va_list ap, void *data);
 static enum evhook_status bookmark_write_hook(va_list ap, void *data);
 
 struct event_hook_info bookmark_hooks[] = {
-	{ "bookmark-delete", bookmark_change_hook, NULL },
-	{ "bookmark-move",   bookmark_change_hook, NULL },
-	{ "bookmark-update", bookmark_change_hook, NULL },
-	{ "periodic-saving", bookmark_write_hook,  NULL },
+	{ "bookmark-delete", 0, bookmark_change_hook, NULL },
+	{ "bookmark-move",   0, bookmark_change_hook, NULL },
+	{ "bookmark-update", 0, bookmark_change_hook, NULL },
+	{ "periodic-saving", 0, bookmark_write_hook,  NULL },
 
 	NULL_EVENT_HOOK_INFO,
 };
@@ -179,8 +178,7 @@ done_bookmarks(struct module *module)
 
 	write_bookmarks();
 	free_bookmarks(&bookmarks, &bookmark_browser.root.child);
-	mem_free_if(bm_last_searched_name);
-	mem_free_if(bm_last_searched_url);
+	free_last_searched_bookmark();
 }
 
 struct module bookmarks_module = struct_module(
@@ -239,6 +237,16 @@ bookmarks_are_dirty(void)
 
 #define check_bookmark_cache(url) (bookmark_cache && (url) && *(url))
 
+static void
+done_bookmark(struct bookmark *bm)
+{
+	done_listbox_item(&bookmark_browser, bm->box_item);
+
+	mem_free(bm->title);
+	mem_free(bm->url);
+	mem_free(bm);
+}
+
 /* Deletes a bookmark. Returns 0 on failure (no such bm), 1 on success. */
 void
 delete_bookmark(struct bookmark *bm)
@@ -262,12 +270,7 @@ delete_bookmark(struct bookmark *bm)
 	del_from_list(bm);
 	bookmarks_set_dirty();
 
-	/* Now wipe the bookmark */
-	done_listbox_item(&bookmark_browser, bm->box_item);
-
-	mem_free(bm->title);
-	mem_free(bm->url);
-	mem_free(bm);
+	done_bookmark(bm);
 }
 
 /* Deletes any bookmarks with no URLs (i.e., folders) and of which
@@ -440,6 +443,22 @@ update_bookmark(struct bookmark *bm, unsigned char *title,
 	return 1;
 }
 
+/* Search for a bookmark with the given title. Search in the given folder
+ * or in the root if folder is NULL. */
+struct bookmark *
+get_bookmark_by_name(struct bookmark *folder, unsigned char *title)
+{
+	struct bookmark *bookmark;
+	struct list_head *lh;
+
+	lh = folder ? &folder->child : &bookmarks;
+
+	foreach (bookmark, *lh)
+		if (!strcmp(bookmark->title, title)) return bookmark;
+
+	return NULL;
+}
+
 /* Search bookmark cache for item matching url. */
 struct bookmark *
 get_bookmark(unsigned char *url)
@@ -537,7 +556,7 @@ bookmark_auto_save_tabs(struct terminal *term)
 }
 
 static void
-bookmark_snapshot()
+bookmark_snapshot(void)
 {
 	struct string folderstring;
 	struct bookmark *folder;
@@ -591,7 +610,7 @@ open_bookmark_folder(struct session *ses, unsigned char *foldername)
 		    || !*bookmark->url)
 			continue;
 
-		uri = get_translated_uri(bookmark->url, 0);
+		uri = get_translated_uri(bookmark->url, NULL);
 		if (!uri) continue;
 
 		/* Save the first bookmark for the current tab */
