@@ -29,6 +29,7 @@
 #include "intl/gettext/libintl.h"
 #include "main/module.h"
 #include "main/object.h"
+#include "main/select.h"
 #include "protocol/date.h"
 #include "protocol/header.h"
 #include "protocol/protocol.h"
@@ -62,6 +63,7 @@ static INIT_LIST_HEAD(c_domains);
 
 static INIT_LIST_HEAD(cookie_servers);
 
+/* Only @set_cookies_dirty may make this nonzero.  */
 static int cookies_dirty = 0;
 
 enum cookies_option {
@@ -200,6 +202,10 @@ done_cookie(struct cookie *c)
 	mem_free(c);
 }
 
+/* The cookie @c can be either in @cookies or in @cookie_queries.
+ * Because changes in @cookie_queries should not affect the cookie
+ * file, this function does not set @cookies_dirty.  Instead, the
+ * caller must do that if appropriate.  */
 void
 delete_cookie(struct cookie *c)
 {
@@ -460,11 +466,12 @@ accept_cookie(struct cookie *cookie)
 				continue;
 
 			delete_cookie(c);
+			/* @set_cookies_dirty will be called below.  */
 		}
 	}
 
 	add_to_list(cookies, cookie);
-	cookies_dirty = 1;
+	set_cookies_dirty();
 
 	/* XXX: This crunches CPU too. --pasky */
 	foreach (cd, c_domains)
@@ -478,9 +485,6 @@ accept_cookie(struct cookie *cookie)
 
 	memcpy(cd->domain, cookie->domain, domain_len + 1);
 	add_to_list(c_domains, cd);
-
-	if (get_cookies_save() && get_cookies_resave())
-		save_cookies();
 }
 
 #if 0
@@ -507,9 +511,6 @@ delete_cookie(struct cookie *c)
 end:
 	del_from_list(c);
 	done_cookie(c);
-
-	if (get_cookies_save() && get_cookies_resave())
-		save_cookies();
 }
 
 
@@ -535,6 +536,7 @@ reject_cookie(void *idp)
 	if (!c)	return;
 
 	delete_cookie(c);
+	set_cookies_dirty(); /* @find_cookie_id doesn't use @cookie_queries */
 }
 
 
@@ -637,7 +639,7 @@ send_cookies(struct uri *uri)
 #endif
 			delete_cookie(c);
 
-			cookies_dirty = 1;
+			set_cookies_dirty();
 			continue;
 		}
 
@@ -658,9 +660,6 @@ send_cookies(struct uri *uri)
 
 	mem_free(path);
 
-	if (cookies_dirty && get_cookies_save() && get_cookies_resave())
-		save_cookies();
-
 	if (!header.length) {
 		done_string(&header);
 		return NULL;
@@ -730,7 +729,7 @@ load_cookies(void) {
 		/* Skip expired cookies if any. */
 		expires = str_to_time_t(members[EXPIRES].pos);
 		if (!expires || expires <= now) {
-			cookies_dirty = 1;
+			set_cookies_dirty();
 			continue;
 		}
 
@@ -761,6 +760,30 @@ load_cookies(void) {
 	fclose(fp);
 }
 
+static void
+resave_cookies_bottom_half(void *always_null)
+{
+	if (get_cookies_save() && get_cookies_resave())
+		save_cookies();	/* checks cookies_dirty */
+}
+
+/* Note that the cookies have been modified, and register a bottom
+ * half for saving them if appropriate.  We use a bottom half so that
+ * if something makes multiple changes and calls this for each change,
+ * the cookies get saved only once at the end.  */
+void
+set_cookies_dirty(void)
+{
+	/* Do not check @cookies_dirty here.  If the previous attempt
+	 * to save cookies failed, @cookies_dirty can still be nonzero
+	 * even though @resave_cookies_bottom_half is no longer in the
+	 * queue.  */
+	cookies_dirty = 1;
+	/* If @resave_cookies_bottom_half is already in the queue,
+	 * @register_bottom_half does nothing.  */
+	register_bottom_half(resave_cookies_bottom_half, NULL);
+}
+
 void
 save_cookies(void) {
 	struct cookie *c;
@@ -801,6 +824,8 @@ init_cookies(struct module *module)
 		load_cookies();
 }
 
+/* Like @delete_cookie, this function does not set @cookies_dirty.
+ * The caller must do that if appropriate.  */
 static void
 free_cookies_list(struct list_head *list)
 {
@@ -821,6 +846,11 @@ done_cookies(struct module *module)
 
 	free_cookies_list(&cookies);
 	free_cookies_list(&cookie_queries);
+	/* If @save_cookies failed above, @cookies_dirty can still be
+	 * nonzero.  Now if @resave_cookies_bottom_half were in the
+	 * queue, it could save the empty @cookies list to the file.
+	 * Prevent that.  */
+	cookies_dirty = 0;
 }
 
 struct module cookies_module = struct_module(
