@@ -1,10 +1,8 @@
-/* $Id: dialogs.h,v 1.5.4.1 2005/02/28 00:44:03 jonas Exp $ */
-
 #ifndef EL__COOKIES_DIALOGS_H
 #define EL__COOKIES_DIALOGS_H
 
 #include "bfu/hierbox.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "terminal/terminal.h"
 #include "util/lists.h"
 
