@@ -1,5 +1,4 @@
 /* Generic support for edit/search historyitem/bookmark dialog */
-/* $Id: edit.c,v 1.81.4.2 2005/04/06 09:11:18 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -12,14 +11,17 @@
 #include "bfu/dialog.h"
 #include "dialogs/edit.h"
 #include "intl/gettext/libintl.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "terminal/terminal.h"
 #include "util/color.h"
 #include "util/memory.h"
 #include "util/string.h"
 
 
-static t_handler_event_status
+/* TODO: Move to bfu/. It has no bussiness to do in dialogs/. --pasky */
+
+
+static widget_handler_status_T
 my_cancel_dialog(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	((void (*)(struct dialog *)) widget_data->widget->data)(dlg_data->dlg);
@@ -41,7 +43,7 @@ do_edit_dialog(struct terminal *term, int intl, unsigned char *title,
 	       const unsigned char *src_name,
 	       const unsigned char *src_url,
 	       struct session *ses, struct dialog_data *parent,
-	       t_done_handler *when_done,
+	       done_handler_T *when_done,
 	       void when_cancel(struct dialog *),
 	       void *done_data,
 	       enum edit_dialog_type dialog_type)
