@@ -1,5 +1,4 @@
 /* Info dialogs */
-/* $Id: info.c,v 1.119.4.7 2005/04/06 09:11:18 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -21,10 +20,11 @@
 #include "dialogs/info.h"
 #include "document/html/renderer.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/select.h"
-#include "modules/version.h"
-#include "sched/connection.h"
-#include "sched/session.h"
+#include "main/select.h"
+#include "main/timer.h"
+#include "main/version.h"
+#include "network/connection.h"
+#include "session/session.h"
 #include "terminal/terminal.h"
 #include "util/conv.h"
 #ifdef DEBUG_MEMLEAK
@@ -61,7 +61,7 @@ menu_keys(struct terminal *term, void *d_, void *xxx)
 	int d = (long) d_;
 
 	/* We scale by main mapping because it has the most actions */
-	int actions[MAIN_ACTIONS] = {
+	action_id_T action_ids[MAIN_ACTIONS] = {
 		ACT_MAIN_MENU,
 		ACT_MAIN_QUIT,
 		ACT_MAIN_MOVE_LINK_NEXT,
@@ -99,16 +99,16 @@ menu_keys(struct terminal *term, void *d_, void *xxx)
 	info->toggle = d;
 
 	if (info->toggle) {
-		int action;
-		enum keymap map;
+		action_id_T action_id;
+		enum keymap_id keymap_id;
 
-		for (action = 0; action < MAIN_ACTIONS - 1; action++) {
-			actions[action] = action + 1;
+		for (action_id = 0; action_id < MAIN_ACTIONS - 1; action_id++) {
+			action_ids[action_id] = action_id + 1;
 		}
 
-		for (map = 0; map < KEYMAP_MAX; map++) {
-			add_actions_to_string(&keys, actions, map, term);
-			if (map + 1 < KEYMAP_MAX)
+		for (keymap_id = 0; keymap_id < KEYMAP_MAX; keymap_id++) {
+			add_actions_to_string(&keys, action_ids, keymap_id, term);
+			if (keymap_id + 1 < KEYMAP_MAX)
 				add_to_string(&keys, "\n\n");
 
 			/* Just a little reminder that the following code takes
@@ -116,14 +116,14 @@ menu_keys(struct terminal *term, void *d_, void *xxx)
 			assert(MAIN_ACTIONS > EDIT_ACTIONS);
 			assert(EDIT_ACTIONS > MENU_ACTIONS);
 
-			if (map == KEYMAP_MAIN) {
-				actions[EDIT_ACTIONS] = ACT_EDIT_NONE;
-			} else if (map == KEYMAP_EDIT) {
-				actions[MENU_ACTIONS] = ACT_MENU_NONE;
+			if (keymap_id == KEYMAP_MAIN) {
+				action_ids[EDIT_ACTIONS] = ACT_EDIT_NONE;
+			} else if (keymap_id == KEYMAP_EDIT) {
+				action_ids[MENU_ACTIONS] = ACT_MENU_NONE;
 			}
 		}
 	} else {
-		add_actions_to_string(&keys, (int *) actions, KEYMAP_MAIN, term);
+		add_actions_to_string(&keys, action_ids, KEYMAP_MAIN, term);
 	}
 
 	msg_box(term, getml(info, NULL), MSGBOX_FREE_TEXT | MSGBOX_SCROLLABLE,
@@ -143,7 +143,7 @@ menu_copying(struct terminal *term, void *xxx, void *xxxx)
 		 	 "\n"
 			 "(C) 1999 - 2002 Mikulas Patocka\n"
 			 "(C) 2001 - 2004 Petr Baudis\n"
-			 "(C) 2002 - 2005 Jonas Fonseca\n"
+			 "(C) 2002 - 2006 Jonas Fonseca\n"
 			 "and others\n"
 			 "\n"
 			 "This program is free software; you can redistribute it "
@@ -158,85 +158,78 @@ static unsigned char *
 get_resource_info(struct terminal *term, void *data)
 {
 	struct string info;
-	int terminal_count = 0, session_count = 0;
-	struct terminal *terminal;
-	struct session *ses;
 	long val;
+	unsigned longlong bigval;
 
 	if (!init_string(&info))
 		return NULL;
 
-	foreach (terminal, terminals)
-		terminal_count++;
-
-	foreach (ses, sessions)
-		session_count++;
-
 #define val_add(text) \
 	add_format_to_string(&info, text, val);
 
 	add_to_string(&info, _("Resources", term));
 	add_to_string(&info, ": ");
 
-	val = select_info(INFO_FILES);
-	val_add(n_("%d handle", "%d handles", val, term));
+	val = get_file_handles_count();
+	val_add(n_("%ld handle", "%ld handles", val, term));
 	add_to_string(&info, ", ");
 
-	val = select_info(INFO_TIMERS);
-	val_add(n_("%d timer", "%d timers", val, term));
+	val = get_timers_count();
+	val_add(n_("%ld timer", "%ld timers", val, term));
 	add_to_string(&info, ".\n");
 
 	add_to_string(&info, _("Connections", term));
 	add_to_string(&info, ": ");
 
-	val = connect_info(INFO_FILES);
-	val_add(n_("%d connection", "%d connections", val, term));
+	val = get_connections_count();
+	val_add(n_("%ld connection", "%ld connections", val, term));
 	add_to_string(&info, ", ");
 
-	val = connect_info(INFO_CONNECTING);
-	val_add(n_("%d connecting", "%d connecting", val, term));
+	val = get_connections_connecting_count();
+	val_add(n_("%ld connecting", "%ld connecting", val, term));
 	add_to_string(&info, ", ");
 
-	val = connect_info(INFO_TRANSFER);
-	val_add(n_("%d transferring", "%d transferring", val, term));
+	val = get_connections_transfering_count();
+	val_add(n_("%ld transferring", "%ld transferring", val, term));
 	add_to_string(&info, ", ");
 
-	val = connect_info(INFO_KEEP);
-	val_add(n_("%d keepalive", "%d keepalive", val, term));
+	val = get_keepalive_connections_count();
+	val_add(n_("%ld keepalive", "%ld keepalive", val, term));
 	add_to_string(&info, ".\n");
 
 	add_to_string(&info, _("Memory cache", term));
 	add_to_string(&info, ": ");
 
-	val = cache_info(INFO_BYTES);
-	val_add(n_("%d byte", "%d bytes", val, term));
+	/* What about just using Kibi/Mebi representation here? --jonas */
+	bigval = get_cache_size();
+	add_format_to_string(&info, n_("%ld byte", "%ld bytes", bigval, term), bigval);
 	add_to_string(&info, ", ");
 
-	val = cache_info(INFO_FILES);
-	val_add(n_("%d file", "%d files", val, term));
+	val = get_cache_entry_count();
+	val_add(n_("%ld file", "%ld files", val, term));
 	add_to_string(&info, ", ");
 
-	val = cache_info(INFO_LOCKED);
-	val_add(n_("%d locked", "%d locked", val, term));
+	val = get_cache_entry_used_count();
+	val_add(n_("%ld in use", "%ld in use", val, term));
 	add_to_string(&info, ", ");
 
-	val = cache_info(INFO_LOADING);
-	val_add(n_("%d loading", "%d loading", val, term));
+	val = get_cache_entry_loading_count();
+	val_add(n_("%ld loading", "%ld loading", val, term));
 	add_to_string(&info, ".\n");
 
 	add_to_string(&info, _("Document cache", term));
 	add_to_string(&info, ": ");
 
-	val = formatted_info(INFO_FILES);
-	val_add(n_("%d formatted", "%d formatted", val, term));
+	val = get_format_cache_size();
+	val_add(n_("%ld formatted", "%ld formatted", val, term));
 	add_to_string(&info, ", ");
 
-	val = formatted_info(INFO_LOCKED);
-	val_add(n_("%d locked", "%d locked", val, term));
+	val = get_format_cache_used_count();
+	val_add(n_("%ld in use", "%ld in use", val, term));
 	add_to_string(&info, ", ");
 
-	val = formatted_info(INFO_TIMERS);
-	val_add(n_("%d refreshing", "%d refreshing", val, term));
+	val = get_format_cache_refresh_count();
+	val_add(n_("%ld refreshing", "%ld refreshing", val, term));
 	add_to_string(&info, ".\n");
 
 	add_to_string(&info, _("Interlinking", term));
@@ -247,12 +240,12 @@ get_resource_info(struct terminal *term, void *data)
 		add_to_string(&info, _("slave terminal", term));
 	add_to_string(&info, ", ");
 
-	val = terminal_count;
-	val_add(n_("%d terminal", "%d terminals", val, term));
+	val = list_size(&terminals);
+	val_add(n_("%ld terminal", "%ld terminals", val, term));
 	add_to_string(&info, ", ");
 
-	val = session_count;
-	val_add(n_("%d session", "%d sessions", val, term));
+	val = list_size(&sessions);
+	val_add(n_("%ld session", "%ld sessions", val, term));
 	add_char_to_string(&info, '.');
 
 #ifdef DEBUG_MEMLEAK
@@ -261,11 +254,11 @@ get_resource_info(struct terminal *term, void *data)
 	add_to_string(&info, ": ");
 
 	val = mem_stats.amount;
-	val_add(n_("%d byte", "%d bytes", val, term));
+	val_add(n_("%ld byte", "%ld bytes", val, term));
 	add_to_string(&info, ", ");
 
 	val = mem_stats.true_amount - mem_stats.amount;
-	val_add(n_("%d byte overhead", "%d bytes overhead", val, term));
+	val_add(n_("%ld byte overhead", "%ld bytes overhead", val, term));
 
 	add_format_to_string(&info, " (%0.2f%%).",
 		(double) (mem_stats.true_amount - mem_stats.amount) / (double) mem_stats.amount * 100);
