@@ -1,5 +1,4 @@
 /* Form history related dialogs */
-/* $Id: dialogs.c,v 1.38.2.3 2005/04/06 09:11:18 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -13,13 +12,14 @@
 #include "formhist/dialogs.h"
 #include "formhist/formhist.h"
 #include "dialogs/edit.h"
+#include "document/forms.h"
 #include "intl/gettext/libintl.h"
+#include "main/object.h"
 #include "protocol/uri.h"
-#include "sched/session.h"
+#include "session/session.h"
 #include "terminal/terminal.h"
 #include "util/lists.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/string.h"
 #include "viewer/text/form.h"
 
@@ -112,8 +112,7 @@ delete_formhist_data(struct listbox_item *item, int last)
 
 	assert(!is_object_used(formhist_data));
 
-	del_from_list(formhist_data);
-	free_form(formhist_data);
+	delete_formhist_item(formhist_data);
 }
 
 static struct listbox_ops_messages formhist_messages = {
@@ -158,7 +157,7 @@ static struct listbox_ops formhist_listbox_ops = {
 	&formhist_messages,
 };
 
-static t_handler_event_status
+static widget_handler_status_T
 push_login_button(struct dialog_data *dlg_data,
 		  struct widget_data *button)
 {
@@ -183,7 +182,7 @@ push_login_button(struct dialog_data *dlg_data,
 	return EVENT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 push_toggle_dontsave_button(struct dialog_data *dlg_data,
 			    struct widget_data *button)
 {
@@ -198,10 +197,10 @@ push_toggle_dontsave_button(struct dialog_data *dlg_data,
 	return EVENT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 push_save_button(struct dialog_data *dlg_data, struct widget_data *button)
 {
-	save_forms_to_file();
+	save_formhist_to_file();
 	return EVENT_PROCESSED;
 }
 
@@ -224,6 +223,6 @@ struct_hierbox_browser(
 void
 formhist_manager(struct session *ses)
 {
-	load_forms_from_file();
+	load_formhist_from_file();
 	hierbox_browser(&formhist_browser, ses);
 }
