@@ -1,5 +1,4 @@
 /* Internal MIME types implementation dialogs */
-/* $Id: dialogs.c,v 1.104.4.2 2005/04/06 09:11:19 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -137,14 +136,14 @@ menu_add_ext(struct terminal *term, void *fcp, void *xxx2)
 
 
 static struct menu_item mi_no_ext[] = {
-	INIT_MENU_ITEM(N_("No extensions"), NULL, ACT_MAIN_NONE, NULL, NULL, FREE_LIST | NO_SELECT),
+	INIT_MENU_ITEM(N_("No extensions"), NULL, ACT_MAIN_NONE, NULL, NULL, NO_SELECT),
 	NULL_MENU_ITEM
 };
 
 void
 menu_list_ext(struct terminal *term, void *fn_, void *xxx)
 {
-	menu_func fn = fn_;
+	menu_func_T fn = fn_;
 	struct list_head *opt_tree = get_opt_tree("mime.extension");
 	struct option *opt;
 	struct menu_item *mi = NULL;
