@@ -1,9 +1,8 @@
-/* $Id: cgi.h,v 1.1 2003/12/01 06:23:46 witekfl Exp $ */
 
 #ifndef EL__PROTOCOL_FILE_CGI_H
 #define EL__PROTOCOL_FILE_CGI_H
 
-#include "sched/connection.h"
+struct connection;
 
 int execute_cgi(struct connection *);
 
