@@ -1,5 +1,4 @@
 /* Internal "http" protocol implementation */
-/* $Id: http.c,v 1.387.2.5 2005/05/01 22:47:55 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,11 +25,13 @@
 #include "cookies/cookies.h"
 #include "intl/charsets.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/connect.h"
-#include "lowlevel/sysname.h"
-#include "modules/module.h"
+#include "main/module.h"
+#include "network/connection.h"
+#include "network/progress.h"
+#include "network/socket.h"
 #include "osdep/ascii.h"
 #include "osdep/osdep.h"
+#include "osdep/sysname.h"
 #include "protocol/auth/auth.h"
 #include "protocol/auth/digest.h"
 #include "protocol/date.h"
@@ -39,10 +40,7 @@
 #include "protocol/http/codes.h"
 #include "protocol/http/http.h"
 #include "protocol/uri.h"
-#include "sched/connection.h"
-#include "sched/session.h"
-#include "ssl/connect.h"
-#include "ssl/ssl.h"
+#include "session/session.h"
 #include "terminal/terminal.h"
 #include "util/base64.h"
 #include "util/conv.h"
@@ -50,6 +48,41 @@
 #include "util/string.h"
 
 
+struct http_version {
+	int major;
+	int minor;
+};
+
+#define HTTP_0_9(x)	 ((x).major == 0 && (x).minor == 9)
+#define HTTP_1_0(x)	 ((x).major == 1 && (x).minor == 0)
+#define HTTP_1_1(x)	 ((x).major == 1 && (x).minor == 1)
+#define PRE_HTTP_1_0(x)  ((x).major < 1)
+#define PRE_HTTP_1_1(x)  (PRE_HTTP_1_0(x) || HTTP_1_0(x))
+#define POST_HTTP_1_0(x) ((x).major > 1 || ((x).major == 1 && (x).minor > 0))
+#define POST_HTTP_1_1(x) ((x).major > 1 || ((x).major == 1 && (x).minor > 1))
+
+
+struct http_connection_info {
+	enum blacklist_flags bl_flags;
+	struct http_version recv_version;
+	struct http_version sent_version;
+
+	int close;
+
+#define LEN_CHUNKED -2 /* == we get data in unknown number of chunks */
+#define LEN_FINISHED 0
+	int length;
+
+	/* Either bytes coming in this chunk yet or "parser state". */
+#define CHUNK_DATA_END	-3
+#define CHUNK_ZERO_SIZE	-2
+#define CHUNK_SIZE	-1
+	int chunk_remaining;
+
+	int code;
+};
+
+
 static struct auth_entry proxy_auth;
 
 static unsigned char *accept_charset = NULL;
@@ -194,10 +227,8 @@ struct module http_protocol_module = struct_module(
 );
 
 
-static void decompress_shutdown(struct connection *);
-
 static void
-done_http()
+done_http(void)
 {
 	mem_free_if(proxy_auth.realm);
 	mem_free_if(proxy_auth.nonce);
@@ -210,7 +241,7 @@ done_http()
 }
 
 static void
-init_accept_charset()
+init_accept_charset(void)
 {
 	struct string ac;
 	unsigned char *cs;
@@ -346,8 +377,9 @@ revstr2num(unsigned char *start, unsigned char *end, int *value)
  * It returns a negative value on error, 0 on success.
  */
 static int
-get_http_code(unsigned char *head, int *code, struct http_version *version)
+get_http_code(struct read_buffer *rb, int *code, struct http_version *version)
 {
+	unsigned char *head = rb->data;
 	unsigned char *start;
 
 	*code = 0;
@@ -402,7 +434,7 @@ get_http_code(unsigned char *head, int *code, struct http_version *version)
 }
 
 static int
-check_http_server_bugs(struct uri *uri, struct http_connection_info *info,
+check_http_server_bugs(struct uri *uri, struct http_connection_info *http,
 		       unsigned char *head)
 {
 	unsigned char *server, **s;
@@ -414,7 +446,7 @@ check_http_server_bugs(struct uri *uri, struct http_connection_info *info,
 	};
 
 	if (!get_opt_bool("protocol.http.bugs.allow_blacklist")
-	    || HTTP_1_0(info->sent_version))
+	    || HTTP_1_0(http->sent_version))
 		return 0;
 
 	server = parse_header(head, "Server", NULL);
@@ -436,37 +468,35 @@ static void
 http_end_request(struct connection *conn, enum connection_state state,
 		 int notrunc)
 {
-	set_connection_state(conn, state);
-	decompress_shutdown(conn);
+	shutdown_connection_stream(conn);
 
-	if (conn->state == S_OK && conn->cached) {
-		if (!notrunc) truncate_entry(conn->cached, conn->from, 1);
-		conn->cached->incomplete = 0;
-		conn->cached->preformatted = 0;
+	if (state == S_OK && conn->cached) {
+		normalize_cache_entry(conn->cached, !notrunc ? conn->from : -1);
 	}
 
 	if (conn->info && !((struct http_connection_info *) conn->info)->close
-	    && (!conn->socket.ssl) /* We won't keep alive ssl connections */
+	    && (!conn->socket->ssl) /* We won't keep alive ssl connections */
 	    && (!get_opt_bool("protocol.http.bugs.post_no_keepalive")
 		|| !conn->uri->post)) {
+		set_connection_state(conn, state);
 		add_keepalive_connection(conn, HTTP_KEEPALIVE_TIMEOUT, NULL);
 	} else {
-		abort_connection(conn);
+		abort_connection(conn, state);
 	}
 }
 
-static void http_send_header(struct connection *);
+static void http_send_header(struct socket *);
 
 void
 http_protocol_handler(struct connection *conn)
 {
 	/* setcstate(conn, S_CONN); */
-	set_connection_timeout(conn);
 
 	if (!has_keepalive_connection(conn)) {
-		make_connection(conn, &conn->socket, http_send_header);
+		make_connection(conn->socket, conn->uri, http_send_header,
+				conn->cache_mode >= CACHE_MODE_FORCE_RELOAD);
 	} else {
-		http_send_header(conn);
+		http_send_header(conn->socket);
 	}
 }
 
@@ -476,65 +506,82 @@ proxy_protocol_handler(struct connection *conn)
 	http_protocol_handler(conn);
 }
 
-static void http_get_header(struct connection *);
-
 #define IS_PROXY_URI(x) ((x)->protocol == PROTOCOL_PROXY)
 
 #define connection_is_https_proxy(conn) \
 	(IS_PROXY_URI((conn)->uri) && (conn)->proxied_uri->protocol == PROTOCOL_HTTPS)
 
-static void
-http_send_header(struct connection *conn)
+struct http_connection_info *
+init_http_connection_info(struct connection *conn, int major, int minor, int close)
 {
-	struct http_connection_info *info;
+	struct http_connection_info *http;
+
+	http = mem_calloc(1, sizeof(*http));
+	if (!http) {
+		http_end_request(conn, S_OUT_OF_MEM, 0);
+		return NULL;
+	}
+
+	http->sent_version.major = major;
+	http->sent_version.minor = minor;
+	http->close = close;
+
+	/* The CGI code uses this too and blacklisting expects a host name. */
+	if (conn->proxied_uri->protocol != PROTOCOL_FILE)
+		http->bl_flags = get_blacklist_flags(conn->proxied_uri);
+
+	if (http->bl_flags & SERVER_BLACKLIST_HTTP10
+	    || get_opt_bool("protocol.http.bugs.http10")) {
+		http->sent_version.major = 1;
+		http->sent_version.minor = 0;
+	}
+
+	/* If called from HTTPS proxy connection the connection info might have
+	 * already been allocated. */
+	mem_free_set(&conn->info, http);
+
+	return http;
+}
+
+static void
+http_send_header(struct socket *socket)
+{
+	struct connection *conn = socket->conn;
+	struct http_connection_info *http;
 	int trace = get_opt_bool("protocol.http.trace");
 	struct string header;
 	unsigned char *post_data = NULL;
-	struct auth_entry *entry;
+	struct auth_entry *entry = NULL;
 	struct uri *uri = conn->proxied_uri; /* Set to the real uri */
 	unsigned char *optstr;
 	int use_connect, talking_to_proxy;
 
-	set_connection_timeout(conn);
-
 	/* Sanity check for a host */
 	if (!uri || !uri->host || !*uri->host || !uri->hostlen) {
 		http_end_request(conn, S_BAD_URL, 0);
 		return;
 	}
 
-	info = mem_calloc(1, sizeof(*info));
-	if (!info) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
-		return;
-	}
-
-	/* If called from HTTPS proxy connection the connection info might have
-	 * already been allocated. */
-	mem_free_set(&conn->info, info);
-
-	info->sent_version.major = 1;
-	info->sent_version.minor = 1;
-	info->bl_flags = get_blacklist_flags(uri);
-
-	if (info->bl_flags & SERVER_BLACKLIST_HTTP10
-	    || get_opt_bool("protocol.http.bugs.http10")) {
-		info->sent_version.major = 1;
-		info->sent_version.minor = 0;
-	}
+	http = init_http_connection_info(conn, 1, 1, 0);
+	if (!http) return;
 
 	if (!init_string(&header)) {
 		http_end_request(conn, S_OUT_OF_MEM, 0);
 		return;
 	}
 
-	talking_to_proxy = IS_PROXY_URI(conn->uri) && !conn->socket.ssl;
-	use_connect = connection_is_https_proxy(conn) && !conn->socket.ssl;
+	talking_to_proxy = IS_PROXY_URI(conn->uri) && !conn->socket->ssl;
+	use_connect = connection_is_https_proxy(conn) && !conn->socket->ssl;
 
 	if (trace) {
 		add_to_string(&header, "TRACE ");
 	} else if (use_connect) {
 		add_to_string(&header, "CONNECT ");
+		/* In CONNECT requests, we send only a subset of the
+		 * headers to the proxy.  See the "CONNECT:" comments
+		 * below.  After the CONNECT request succeeds, we
+		 * negotiate TLS with the real server and make a new
+		 * HTTP request that includes all the headers.  */
 	} else if (uri->post) {
 		add_to_string(&header, "POST ");
 		conn->unrestartable = 1;
@@ -550,7 +597,7 @@ http_send_header(struct connection *conn)
 		/* Add port if it was specified or the default port */
 		add_uri_to_string(&header, uri, URI_HTTP_CONNECT);
 	} else {
-		if (connection_is_https_proxy(conn) && conn->socket.ssl) {
+		if (connection_is_https_proxy(conn) && conn->socket->ssl) {
 			add_url_to_http_string(&header, uri, URI_DATA);
 
 		} else if (talking_to_proxy) {
@@ -562,23 +609,27 @@ http_send_header(struct connection *conn)
 	}
 
 	add_to_string(&header, " HTTP/");
-	add_long_to_string(&header, info->sent_version.major);
+	add_long_to_string(&header, http->sent_version.major);
 	add_char_to_string(&header, '.');
-	add_long_to_string(&header, info->sent_version.minor);
+	add_long_to_string(&header, http->sent_version.minor);
 	add_crlf_to_string(&header);
 
+	/* CONNECT: Sending a Host header seems pointless as the same
+	 * information is already in the CONNECT line.  It's harmless
+	 * though and Mozilla does it too.  */
 	add_to_string(&header, "Host: ");
 	add_uri_to_string(&header, uri, URI_HTTP_HOST);
 	add_crlf_to_string(&header);
 
+	/* CONNECT: Proxy-Authorization is intended to be seen by the proxy.  */
 	if (talking_to_proxy) {
 		unsigned char *user = get_opt_str("protocol.http.proxy.user");
 		unsigned char *passwd = get_opt_str("protocol.http.proxy.passwd");
 
 		if (proxy_auth.digest) {
 			unsigned char *response;
-			int userlen = int_min(strlen(user), HTTP_AUTH_USER_MAXLEN - 1);
-			int passwordlen = int_min(strlen(passwd), HTTP_AUTH_PASSWORD_MAXLEN - 1);
+			int userlen = int_min(strlen(user), AUTH_USER_MAXLEN - 1);
+			int passwordlen = int_min(strlen(passwd), AUTH_PASSWORD_MAXLEN - 1);
 
 			if (userlen)
 				memcpy(proxy_auth.user, user, userlen);
@@ -618,6 +669,9 @@ http_send_header(struct connection *conn)
 		}
 	}
 
+	/* CONNECT: User-Agent does not reveal anything about the
+	 * resource we're fetching, and it may help the proxy return
+	 * better error messages.  */
 	optstr = get_opt_str("protocol.http.user_agent");
 	if (*optstr && strcmp(optstr, " ")) {
 		unsigned char *ustr, ts[64] = "";
@@ -643,33 +697,47 @@ http_send_header(struct connection *conn)
 		add_crlf_to_string(&header);
 	}
 
-	switch (get_opt_int("protocol.http.referer.policy")) {
-		case REFERER_NONE:
-			/* oh well */
-			break;
+	/* CONNECT: Referer probably is a secret page in the HTTPS
+	 * server, so don't reveal it to the proxy.  */ 
+	if (!use_connect) {
+		switch (get_opt_int("protocol.http.referer.policy")) {
+			case REFERER_NONE:
+				/* oh well */
+				break;
 
-		case REFERER_FAKE:
-			optstr = get_opt_str("protocol.http.referer.fake");
-			if (!optstr[0]) break;
-			add_to_string(&header, "Referer: ");
-			add_to_string(&header, optstr);
-			add_crlf_to_string(&header);
-			break;
+			case REFERER_FAKE:
+				optstr = get_opt_str("protocol.http.referer.fake");
+				if (!optstr[0]) break;
+				add_to_string(&header, "Referer: ");
+				add_to_string(&header, optstr);
+				add_crlf_to_string(&header);
+				break;
 
-		case REFERER_TRUE:
-			if (!conn->referrer) break;
-			add_to_string(&header, "Referer: ");
-			add_url_to_http_string(&header, conn->referrer, URI_HTTP_REFERRER);
-			add_crlf_to_string(&header);
-			break;
+			case REFERER_TRUE:
+				if (!conn->referrer) break;
+				add_to_string(&header, "Referer: ");
+				add_url_to_http_string(&header, conn->referrer, URI_HTTP_REFERRER);
+				add_crlf_to_string(&header);
+				break;
 
-		case REFERER_SAME_URL:
-			add_to_string(&header, "Referer: ");
-			add_url_to_http_string(&header, uri, URI_HTTP_REFERRER);
-			add_crlf_to_string(&header);
-			break;
+			case REFERER_SAME_URL:
+				add_to_string(&header, "Referer: ");
+				add_url_to_http_string(&header, uri, URI_HTTP_REFERRER);
+				add_crlf_to_string(&header);
+				break;
+		}
 	}
 
+	/* CONNECT: Do send all Accept* headers to the CONNECT proxy,
+	 * because they do not reveal anything about the resource
+	 * we're going to request via TLS, and they may affect the
+	 * error message if the CONNECT request fails.
+	 *
+	 * If ELinks is ever changed to vary its Accept headers based
+	 * on what it intends to do with the returned resource, e.g.
+	 * sending "Accept: text/css" when it wants an external
+	 * stylesheet, then it should do that only in the inner GET
+	 * and not in the outer CONNECT.  */
 	add_to_string(&header, "Accept: */*");
 	add_crlf_to_string(&header);
 
@@ -700,7 +768,7 @@ http_send_header(struct connection *conn)
 		init_accept_charset();
 	}
 
-	if (!(info->bl_flags & SERVER_BLACKLIST_NO_CHARSET)
+	if (!(http->bl_flags & SERVER_BLACKLIST_NO_CHARSET)
 	    && !get_opt_bool("protocol.http.bugs.accept_charset")
 	    && accept_charset) {
 		add_to_string(&header, accept_charset);
@@ -712,7 +780,7 @@ http_send_header(struct connection *conn)
 		add_to_string(&header, optstr);
 		add_crlf_to_string(&header);
 	}
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 	else if (get_opt_bool("protocol.http.accept_ui_language")) {
 		unsigned char *code = language_to_iso639(current_language);
 
@@ -724,8 +792,13 @@ http_send_header(struct connection *conn)
 	}
 #endif
 
+	/* CONNECT: Proxy-Connection is intended to be seen by the
+	 * proxy.  If the CONNECT request succeeds, then the proxy
+	 * will forward the remainder of the TCP connection to the
+	 * origin server, and Proxy-Connection does not matter; but
+	 * if the request fails, then Proxy-Connection may matter.  */
 	/* FIXME: What about post-HTTP/1.1?? --Zas */
-	if (HTTP_1_1(info->sent_version)) {
+	if (HTTP_1_1(http->sent_version)) {
 		if (!IS_PROXY_URI(conn->uri)) {
 			add_to_string(&header, "Connection: ");
 		} else {
@@ -740,7 +813,9 @@ http_send_header(struct connection *conn)
 		add_crlf_to_string(&header);
 	}
 
-	if (conn->cached) {
+ 	/* CONNECT: Do not tell the proxy anything we have cached
+ 	 * about the resource.  */
+ 	if (!use_connect && conn->cached) {
 		if (!conn->cached->incomplete && conn->cached->head && conn->cached->last_modified
 		    && conn->cache_mode <= CACHE_MODE_CHECK_IF_MODIFIED) {
 			add_to_string(&header, "If-Modified-Since: ");
@@ -749,6 +824,8 @@ http_send_header(struct connection *conn)
 		}
 	}
 
+	/* CONNECT: Let's send cache control headers to the proxy too;
+	 * they may affect DNS caching.  */
 	if (conn->cache_mode >= CACHE_MODE_FORCE_RELOAD) {
 		add_to_string(&header, "Pragma: no-cache");
 		add_crlf_to_string(&header);
@@ -756,17 +833,22 @@ http_send_header(struct connection *conn)
 		add_crlf_to_string(&header);
 	}
 
-	if (conn->from || (conn->progress.start > 0)) {
+	/* CONNECT: Do not reveal byte ranges to the proxy.  It can't
+	 * do anything good with that information anyway.  */
+	if (!use_connect && (conn->from || conn->progress->start > 0)) {
 		/* conn->from takes precedence. conn->progress.start is set only the first
 		 * time, then conn->from gets updated and in case of any retries
 		 * etc we have everything interesting in conn->from already. */
 		add_to_string(&header, "Range: bytes=");
-		add_long_to_string(&header, conn->from ? conn->from : conn->progress.start);
+		add_long_to_string(&header, conn->from ? conn->from : conn->progress->start);
 		add_char_to_string(&header, '-');
 		add_crlf_to_string(&header);
 	}
 
-	entry = find_auth(uri);
+ 	/* CONNECT: The Authorization header is for the origin server only.  */
+ 	if (!use_connect) {
+		entry = find_auth(uri);
+ 	}
 	if (entry) {
 		if (entry->digest) {
 			unsigned char *response;
@@ -806,7 +888,8 @@ http_send_header(struct connection *conn)
 		}
 	}
 
-	if (uri->post) {
+	/* CONNECT: Any POST data is for the origin server only.  */
+	if (!use_connect && uri->post) {
 		/* We search for first '\n' in uri->post to get content type
 		 * as set by get_form_uri(). This '\n' is dropped if any
 		 * and replaced by correct '\r\n' termination here. */
@@ -825,7 +908,8 @@ http_send_header(struct connection *conn)
 	}
 
 #ifdef CONFIG_COOKIES
-	{
+	/* CONNECT: Cookies are for the origin server only.  */
+	if (!use_connect) {
 		struct string *cookies = send_cookies(uri);
 
 		if (cookies) {
@@ -839,12 +923,17 @@ http_send_header(struct connection *conn)
 
 	add_crlf_to_string(&header);
 
+	/* CONNECT: Any POST data is for the origin server only.
+	 * This was already checked above and post_data is NULL
+	 * in that case.  Verified with an assertion below.  */
 	if (post_data) {
 #define POST_BUFFER_SIZE 4096
 		unsigned char *post = post_data;
 		unsigned char buffer[POST_BUFFER_SIZE];
 		int n = 0;
 
+		assert(!use_connect); /* see comment above */
+
 		while (post[0] && post[1]) {
 			int h1, h2;
 
@@ -869,11 +958,9 @@ http_send_header(struct connection *conn)
 #undef POST_BUFFER_SIZE
 	}
 
-	write_to_socket(conn, &conn->socket, header.source, header.length,
-			http_get_header);
+	request_from_socket(socket, header.source, header.length, S_SENT,
+			    SOCKET_END_ONCLOSE, http_got_header);
 	done_string(&header);
-
-	set_connection_state(conn, S_SENT);
 }
 
 
@@ -882,7 +969,7 @@ http_send_header(struct connection *conn)
  * back to the world as the return value and its length is stored into
  * @new_len.
  *
- * In this function, value of either info->chunk_remaining or info->length is
+ * In this function, value of either http->chunk_remaining or http->length is
  * being changed (it depends on if chunked mode is used or not).
  *
  * Note that the function is still a little esotheric for me. Don't take it
@@ -895,7 +982,7 @@ static unsigned char *
 decompress_data(struct connection *conn, unsigned char *data, int len,
 		int *new_len)
 {
-	struct http_connection_info *info = conn->info;
+	struct http_connection_info *http = conn->info;
 	/* to_read is number of bytes to be read from the decoder. It is 65536
 	 * (then we are just emptying the decoder buffer as we finished the walk
 	 * through the incoming stream already) or PIPE_BUF / 2 (when we are
@@ -910,8 +997,8 @@ decompress_data(struct connection *conn, unsigned char *data, int len,
 	int *length_of_block;
 	unsigned char *output = NULL;
 
-	length_of_block = (info->length == LEN_CHUNKED ? &info->chunk_remaining
-						       : &info->length);
+	length_of_block = (http->length == LEN_CHUNKED ? &http->chunk_remaining
+						       : &http->length);
 
 #define BIG_READ 65536
 	if (!*length_of_block) {
@@ -948,12 +1035,12 @@ decompress_data(struct connection *conn, unsigned char *data, int len,
 				data += written;
 				len -= written;
 
-				/* In non-keep-alive connections info->length == -1, so the test below */
+				/* In non-keep-alive connections http->length == -1, so the test below */
 				if (*length_of_block > 0)
 					*length_of_block -= written;
-				/* info->length is 0 at the end of block for all modes: keep-alive,
+				/* http->length is 0 at the end of block for all modes: keep-alive,
 				 * non-keep-alive and chunked */
-				if (!info->length) {
+				if (!http->length) {
 					/* That's all, folks - let's finish this. */
 					to_read = BIG_READ;
 				} else if (!len) {
@@ -990,40 +1077,27 @@ decompress_data(struct connection *conn, unsigned char *data, int len,
 			*new_len = 0;
 			break; /* Loop prevention (bug 517), is this correct ? --Zas */
 		}
-	} while (!(!len && did_read != to_read));
+	} while (len || did_read == BIG_READ);
 
-	decompress_shutdown(conn);
+	shutdown_connection_stream(conn);
 	return output;
 }
 
-/* FIXME: Unfortunately, we duplicate this in free_connection_data(). */
-static void
-decompress_shutdown(struct connection *conn)
-{
-	if (conn->stream) {
-		close_encoded(conn->stream);
-		conn->stream = NULL;
-	}
-	if (conn->stream_pipes[1] >= 0)
-		close(conn->stream_pipes[1]);
-	conn->stream_pipes[0] = conn->stream_pipes[1] = -1;
-}
-
 static int
 is_line_in_buffer(struct read_buffer *rb)
 {
 	int l;
 
-	for (l = 0; l < rb->len; l++) {
+	for (l = 0; l < rb->length; l++) {
 		unsigned char a0 = rb->data[l];
 
 		if (a0 == ASCII_LF)
 			return l + 1;
 		if (a0 == ASCII_CR) {
 			if (rb->data[l + 1] == ASCII_LF
-			    && l < rb->len - 1)
+			    && l < rb->length - 1)
 				return l + 2;
-			if (l == rb->len - 1)
+			if (l == rb->length - 1)
 				return 0;
 		}
 		if (a0 < ' ')
@@ -1032,27 +1106,27 @@ is_line_in_buffer(struct read_buffer *rb)
 	return 0;
 }
 
-static void read_http_data(struct connection *conn, struct read_buffer *rb);
+static void read_http_data(struct socket *socket, struct read_buffer *rb);
 
 static void
 read_more_http_data(struct connection *conn, struct read_buffer *rb,
                     int already_got_anything)
 {
-	read_from_socket(conn, &conn->socket, rb, read_http_data);
-	if (already_got_anything)
-		set_connection_state(conn, S_TRANS);
+	enum connection_state state = already_got_anything ? S_TRANS : conn->state;
+
+	read_from_socket(conn->socket, rb, state, read_http_data);
 }
 
 static void
 read_http_data_done(struct connection *conn)
 {
-	struct http_connection_info *info = conn->info;
+	struct http_connection_info *http = conn->info;
 
 	/* There's no content but an error so just print
 	 * that instead of nothing. */
 	if (!conn->from) {
-		if (info->http_code >= 400) {
-			http_error_document(conn, info->http_code);
+		if (http->code >= 400) {
+			http_error_document(conn, http->code);
 
 		} else {
 			/* This is not an error, thus fine. No need generate any
@@ -1074,7 +1148,7 @@ read_http_data_done(struct connection *conn)
 static int
 read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 {
-	struct http_connection_info *info = conn->info;
+	struct http_connection_info *http = conn->info;
 	int total_data_len = 0;
 
 	while (1) {
@@ -1084,7 +1158,7 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 		 * aklkjadslkfjalkfjlkajkljfdkljdsfkljdf*1234\r\n
 		 * 0\r\n
 		 * \r\n */
-		if (info->chunk_remaining == CHUNK_DATA_END) {
+		if (http->chunk_remaining == CHUNK_DATA_END) {
 			int l = is_line_in_buffer(rb);
 
 			if (l) {
@@ -1102,7 +1176,7 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 				continue;
 			}
 
-		} else if (info->chunk_remaining == CHUNK_SIZE) {
+		} else if (http->chunk_remaining == CHUNK_SIZE) {
 			int l = is_line_in_buffer(rb);
 
 			if (l) {
@@ -1123,9 +1197,9 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 
 				/* Remove everything to the EOLN. */
 				kill_buffer_data(rb, l);
-				info->chunk_remaining = n;
-				if (!info->chunk_remaining)
-					info->chunk_remaining = CHUNK_ZERO_SIZE;
+				http->chunk_remaining = n;
+				if (!http->chunk_remaining)
+					http->chunk_remaining = CHUNK_ZERO_SIZE;
 				continue;
 			}
 
@@ -1133,13 +1207,13 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 			unsigned char *data;
 			int data_len;
 			int len;
-			int zero = (info->chunk_remaining == CHUNK_ZERO_SIZE);
+			int zero = (http->chunk_remaining == CHUNK_ZERO_SIZE);
 
-			if (zero) info->chunk_remaining = 0;
-			len = info->chunk_remaining;
+			if (zero) http->chunk_remaining = 0;
+			len = http->chunk_remaining;
 
 			/* Maybe everything necessary didn't come yet.. */
-			int_upper_bound(&len, rb->len);
+			int_upper_bound(&len, rb->length);
 			conn->received += len;
 
 			data = decompress_data(conn, rb->data, len, &data_len);
@@ -1160,24 +1234,24 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 				 * chunk, we finished decompression just now
 				 * and now we can happily finish reading this
 				 * stuff. */
-				info->chunk_remaining = CHUNK_DATA_END;
+				http->chunk_remaining = CHUNK_DATA_END;
 				continue;
 			}
 
-			if (!info->chunk_remaining && rb->len > 0) {
+			if (!http->chunk_remaining && rb->length > 0) {
 				/* Eat newline succeeding each chunk. */
 				if (rb->data[0] == ASCII_LF) {
 					kill_buffer_data(rb, 1);
 				} else {
 					if (rb->data[0] != ASCII_CR
-					    || (rb->len >= 2
+					    || (rb->length >= 2
 						&& rb->data[1] != ASCII_LF)) {
 						return -1;
 					}
-					if (rb->len < 2) break;
+					if (rb->length < 2) break;
 					kill_buffer_data(rb, 2);
 				}
-				info->chunk_remaining = CHUNK_SIZE;
+				http->chunk_remaining = CHUNK_SIZE;
 				continue;
 			}
 		}
@@ -1192,14 +1266,14 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 static int
 read_normal_http_data(struct connection *conn, struct read_buffer *rb)
 {
-	struct http_connection_info *info = conn->info;
+	struct http_connection_info *http = conn->info;
 	unsigned char *data;
 	int data_len;
-	int len = rb->len;
+	int len = rb->length;
 
-	if (info->length >= 0 && info->length < len) {
+	if (http->length >= 0 && http->length < len) {
 		/* We won't read more than we have to go. */
-		len = info->length;
+		len = http->length;
 	}
 
 	conn->received += len;
@@ -1215,7 +1289,7 @@ read_normal_http_data(struct connection *conn, struct read_buffer *rb)
 
 	kill_buffer_data(rb, len);
 
-	if (!info->length && rb->close == READ_BUFFER_RETRY_ONCLOSE) {
+	if (!http->length && conn->socket->state == SOCKET_RETRY_ONCLOSE) {
 		return 2;
 	}
 
@@ -1223,24 +1297,23 @@ read_normal_http_data(struct connection *conn, struct read_buffer *rb)
 }
 
 static void
-read_http_data(struct connection *conn, struct read_buffer *rb)
+read_http_data(struct socket *socket, struct read_buffer *rb)
 {
-	struct http_connection_info *info = conn->info;
+	struct connection *conn = socket->conn;
+	struct http_connection_info *http = conn->info;
 	int ret;
 
-	set_connection_timeout(conn);
-
-	if (rb->close == READ_BUFFER_END) {
-		if (conn->content_encoding && info->length == -1) {
+	if (socket->state == SOCKET_CLOSED) {
+		if (conn->content_encoding && http->length == -1) {
 			/* Flush decompression first. */
-			info->length = 0;
+			http->length = 0;
 		} else {
 			read_http_data_done(conn);
 			return;
 		}
 	}
 
-	if (info->length != LEN_CHUNKED) {
+	if (http->length != LEN_CHUNKED) {
 		ret = read_normal_http_data(conn, rb);
 
 	} else {
@@ -1248,9 +1321,6 @@ read_http_data(struct connection *conn, struct read_buffer *rb)
 	}
 
 	switch (ret) {
-	case -1:
-		abort_conn_with_state(conn, S_HTTP_ERROR);
-		break;
 	case 0:
 		read_more_http_data(conn, rb, 0);
 		break;
@@ -1261,7 +1331,8 @@ read_http_data(struct connection *conn, struct read_buffer *rb)
 		read_http_data_done(conn);
 		break;
 	default:
-		INTERNAL("Unexpected return value: %d", ret);
+		assertm(ret == -1, "Unexpected return value: %d", ret);
+		abort_connection(conn, S_HTTP_ERROR);
 	}
 }
 
@@ -1276,10 +1347,10 @@ get_header(struct read_buffer *rb)
 	/* XXX: We will have to do some guess about whether an HTTP header is
 	 * coming or not, in order to support HTTP/0.9 reply correctly. This
 	 * means a little code duplication with get_http_code(). --pasky */
-	if (rb->len > 4 && strncasecmp(rb->data, "HTTP/", 5))
+	if (rb->length > 4 && strncasecmp(rb->data, "HTTP/", 5))
 		return -2;
 
-	for (i = 0; i < rb->len; i++) {
+	for (i = 0; i < rb->length; i++) {
 		unsigned char a0 = rb->data[i];
 		unsigned char a1 = rb->data[i + 1];
 
@@ -1288,9 +1359,9 @@ get_header(struct read_buffer *rb)
 			continue;
 		}
 		if (a0 == ASCII_LF && a1 == ASCII_LF
-		    && i < rb->len - 1)
+		    && i < rb->length - 1)
 			return i + 2;
-		if (a0 == ASCII_CR && i < rb->len - 3) {
+		if (a0 == ASCII_CR && i < rb->length - 3) {
 			if (a1 == ASCII_CR) continue;
 			if (a1 != ASCII_LF) return -1;
 			if (rb->data[i + 2] == ASCII_CR) {
@@ -1342,9 +1413,10 @@ check_http_authentication(struct uri *uri, unsigned char *header,
 
 
 void
-http_got_header(struct connection *conn, struct read_buffer *rb)
+http_got_header(struct socket *socket, struct read_buffer *rb)
 {
-	struct http_connection_info *info = conn->info;
+	struct connection *conn = socket->conn;
+	struct http_connection_info *http = conn->info;
 	unsigned char *head;
 #ifdef CONFIG_COOKIES
 	unsigned char *cookie, *ch;
@@ -1356,37 +1428,34 @@ http_got_header(struct connection *conn, struct read_buffer *rb)
 	int a, h = 200;
 	int cf;
 
-	set_connection_timeout(conn);
-
-	if (rb->close == READ_BUFFER_END) {
+	if (socket->state == SOCKET_CLOSED) {
 		if (!conn->tries && uri->host) {
-			if (info->bl_flags & SERVER_BLACKLIST_NO_CHARSET) {
+			if (http->bl_flags & SERVER_BLACKLIST_NO_CHARSET) {
 				del_blacklist_entry(uri, SERVER_BLACKLIST_NO_CHARSET);
 			} else {
 				add_blacklist_entry(uri, SERVER_BLACKLIST_NO_CHARSET);
 				conn->tries = -1;
 			}
 		}
-		retry_conn_with_state(conn, S_CANT_READ);
+		retry_connection(conn, S_CANT_READ);
 		return;
 	}
-	rb->close = READ_BUFFER_RETRY_ONCLOSE;
+	socket->state = SOCKET_RETRY_ONCLOSE;
 
 again:
 	a = get_header(rb);
 	if (a == -1) {
-		abort_conn_with_state(conn, S_HTTP_ERROR);
+		abort_connection(conn, S_HTTP_ERROR);
 		return;
 	}
 	if (!a) {
-		read_from_socket(conn, &conn->socket, rb, http_got_header);
-		set_connection_state(conn, state);
+		read_from_socket(conn->socket, rb, state, http_got_header);
 		return;
 	}
 	if (a == -2) a = 0;
-	if ((a && get_http_code(rb->data, &h, &version))
+	if ((a && get_http_code(rb, &h, &version))
 	    || h == 101) {
-		abort_conn_with_state(conn, S_HTTP_ERROR);
+		abort_connection(conn, S_HTTP_ERROR);
 		return;
 	}
 
@@ -1399,13 +1468,13 @@ again:
 	head = (a ? memacpy(rb->data, a)
 		  : stracpy("\r\nContent-Type: text/html\r\n"));
 	if (!head) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
-	if (check_http_server_bugs(uri, info, head)) {
+	if (check_http_server_bugs(uri, http, head)) {
 		mem_free(head);
-		retry_conn_with_state(conn, S_RESTART);
+		retry_connection(conn, S_RESTART);
 		return;
 	}
 
@@ -1424,7 +1493,7 @@ again:
 			if (h2 >= 100 && h2 < 600) h = h2;
 			if (h == 101) {
 				mem_free(head);
-				abort_conn_with_state(conn, S_HTTP_ERROR);
+				abort_connection(conn, S_HTTP_ERROR);
 				return;
 			}
 		}
@@ -1438,7 +1507,7 @@ again:
 		mem_free(cookie);
 	}
 #endif
-	info->http_code = h;
+	http->code = h;
 
 	if (h == 100) {
 		mem_free(head);
@@ -1448,7 +1517,7 @@ again:
 	}
 	if (h < 200) {
 		mem_free(head);
-		abort_conn_with_state(conn, S_HTTP_ERROR);
+		abort_connection(conn, S_HTTP_ERROR);
 		return;
 	}
 	if (h == 304) {
@@ -1461,19 +1530,13 @@ again:
 		http_end_request(conn, S_HTTP_204, 0);
 		return;
 	}
-	if (h == 200 && connection_is_https_proxy(conn) && !conn->socket.ssl) {
+	if (h == 200 && connection_is_https_proxy(conn) && !conn->socket->ssl) {
 #ifdef CONFIG_SSL
 		mem_free(head);
-		conn->conn_info = init_connection_info(uri, &conn->socket,
-						       http_send_header);
-		if (!conn->conn_info) {
-			abort_conn_with_state(conn, S_OUT_OF_MEM);
-			return;
-		}
-
-		if (ssl_connect(conn, &conn->socket) == -1) return;
+		socket->need_ssl = 1;
+		complete_connect_socket(socket, uri, http_send_header);
 #else
-		abort_conn_with_state(conn, S_SSL_ERROR);
+		abort_connection(conn, S_SSL_ERROR);
 #endif
 		return;
 	}
@@ -1481,7 +1544,7 @@ again:
 	conn->cached = get_cache_entry(conn->uri);
 	if (!conn->cached) {
 		mem_free(head);
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 	mem_free_set(&conn->cached->head, head);
@@ -1504,7 +1567,7 @@ again:
 			mem_free(d);
 
 			if (expires && cached->cache_mode != CACHE_MODE_NEVER) {
-				cached->max_age = expires;
+				timeval_from_seconds(&cached->max_age, expires);
 				cached->expire = 1;
 			}
 		}
@@ -1530,9 +1593,12 @@ again:
 
 				if (pos) {
 					/* Grab the number of seconds. */
-					time_t max_age = str_to_ttime(pos + 8);
+					timeval_T max_age;
+
+					timeval_from_seconds(&max_age, atol(pos + 8));
+					timeval_now(&cached->max_age);
+					timeval_add_interval(&cached->max_age, &max_age);
 
-					cached->max_age = max_age + time(NULL);
 					cached->expire = 1;
 				}
 			}
@@ -1541,13 +1607,6 @@ again:
 		}
 	}
 
-#ifdef CONFIG_SSL
-	/* TODO: Move this to some more generic place like lowlevel/connect.c
-	 * or sched/connection.c when other protocols will need it. --jonas */
-	if (conn->socket.ssl)
-		mem_free_set(&conn->cached->ssl_info, get_ssl_connection_cipher(conn));
-#endif
-
 	/* XXX: Is there some reason why NOT to follow the Location header
 	 * for any status? If the server didn't mean it, it wouldn't send
 	 * it, after all...? --pasky */
@@ -1616,16 +1675,16 @@ again:
 	}
 
 	kill_buffer_data(rb, a);
-	info->close = 0;
-	info->length = -1;
-	info->recv_version = version;
+	http->close = 0;
+	http->length = -1;
+	http->recv_version = version;
 
 	if ((d = parse_header(conn->cached->head, "Connection", NULL))
 	     || (d = parse_header(conn->cached->head, "Proxy-Connection", NULL))) {
-		if (!strcasecmp(d, "close")) info->close = 1;
+		if (!strcasecmp(d, "close")) http->close = 1;
 		mem_free(d);
 	} else if (PRE_HTTP_1_1(version)) {
-		info->close = 1;
+		http->close = 1;
 	}
 
 	cf = conn->from;
@@ -1646,10 +1705,10 @@ again:
 		mem_free(d);
 	}
 	if (cf && !conn->from && !conn->unrestartable) conn->unrestartable = 1;
-	if ((conn->progress.start <= 0 && conn->from > cf) || conn->from < 0) {
+	if ((conn->progress->start <= 0 && conn->from > cf) || conn->from < 0) {
 		/* We don't want this if conn->progress.start because then conn->from will
 		 * be probably value of conn->progress.start, while cf is 0. */
-		abort_conn_with_state(conn, S_HTTP_ERROR);
+		abort_connection(conn, S_HTTP_ERROR);
 		return;
 	}
 
@@ -1664,11 +1723,11 @@ again:
 	}
 #endif
 
-	if (conn->progress.start >= 0) {
+	if (conn->progress->start >= 0) {
 		/* Update to the real value which we've got from Content-Range. */
-		conn->progress.seek = conn->from;
+		conn->progress->seek = conn->from;
 	}
-	conn->progress.start = conn->from;
+	conn->progress->start = conn->from;
 
 	d = parse_header(conn->cached->head, "Content-Length", NULL);
 	if (d) {
@@ -1679,8 +1738,8 @@ again:
 		l = strtol(d, (char **) &ep, 10);
 
 		if (!errno && !*ep && l >= 0) {
-			if (!info->close || POST_HTTP_1_0(version))
-				info->length = l;
+			if (!http->close || POST_HTTP_1_0(version))
+				http->length = l;
 			conn->est_length = conn->from + l;
 		}
 		mem_free(d);
@@ -1702,12 +1761,12 @@ again:
 	d = parse_header(conn->cached->head, "Transfer-Encoding", NULL);
 	if (d) {
 		if (!strcasecmp(d, "chunked")) {
-			info->length = LEN_CHUNKED;
-			info->chunk_remaining = CHUNK_SIZE;
+			http->length = LEN_CHUNKED;
+			http->chunk_remaining = CHUNK_SIZE;
 		}
 		mem_free(d);
 	}
-	if (!info->close && info->length == -1) info->close = 1;
+	if (!http->close && http->length == -1) http->close = 1;
 
 	d = parse_header(conn->cached->head, "Last-Modified", NULL);
 	if (d) {
@@ -1716,7 +1775,7 @@ again:
 			if (conn->from) {
 				conn->from = 0;
 				mem_free(d);
-				retry_conn_with_state(conn, S_MODIFIED);
+				retry_connection(conn, S_MODIFIED);
 				return;
 			}
 		}
@@ -1748,7 +1807,7 @@ again:
 				if (conn->from) {
 					conn->from = 0;
 					mem_free(d);
-					retry_conn_with_state(conn, S_MODIFIED);
+					retry_connection(conn, S_MODIFIED);
 					return;
 				}
 			}
@@ -1791,20 +1850,9 @@ again:
 		conn->cached->encoding_info = stracpy(get_encoding_name(conn->content_encoding));
 	}
 
-	if (info->length == -1
-	    || (PRE_HTTP_1_1(info->recv_version) && info->close))
-		rb->close = READ_BUFFER_END_ONCLOSE;
+	if (http->length == -1
+	    || (PRE_HTTP_1_1(http->recv_version) && http->close))
+		socket->state = SOCKET_END_ONCLOSE;
 
-	read_http_data(conn, rb);
-}
-
-static void
-http_get_header(struct connection *conn)
-{
-	struct read_buffer *rb = alloc_read_buffer(conn);
-
-	if (!rb) return;
-	set_connection_timeout(conn);
-	rb->close = READ_BUFFER_END_ONCLOSE;
-	read_from_socket(conn, &conn->socket, rb, http_got_header);
+	read_http_data(socket, rb);
 }
