@@ -1,5 +1,4 @@
 /* Terminal screen drawing routines. */
-/* $Id: screen.c,v 1.152.6.5 2005/04/05 21:08:43 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -206,7 +205,7 @@ update_screen_driver(struct screen_driver *driver, struct option *term_spec)
 	}
 
 	if (utf8_io) {
-		driver->charsets[0] = get_opt_int_tree(term_spec, "charset");
+		driver->charsets[0] = get_opt_codepage_tree(term_spec, "charset");
 		if (driver->type == TERM_LINUX) {
 			if (get_opt_bool_tree(term_spec, "restrict_852"))
 				driver->frame = frame_restrict;
@@ -298,10 +297,7 @@ get_screen_driver(struct terminal *term)
 		if (memcmp(driver->name, name, len + 1)) continue;
 
 		/* Some simple probably useless MRU ;) */
-		if (driver != active_screen_drivers.next) {
-			del_from_list(driver);
-			add_to_list(active_screen_drivers, driver);
-		}
+		move_to_top_of_list(active_screen_drivers, driver);
 
 		return driver;
 	}
@@ -321,26 +317,25 @@ done_screen_drivers(void)
 static inline struct string *
 add_cursor_move_to_string(struct string *screen, int y, int x)
 {
-	/* 28 chars for both of the @y and @x numbers should be enough. */
-	unsigned char code[32];
-	int length = 2;
-	int ret;
+#define CURSOR_NUM_LEN 10 /* 10 chars for @y and @x numbers should be more than enough. */
+	unsigned char code[4 + 2 * CURSOR_NUM_LEN + 1];
+	unsigned int length = 2;
 
 	code[0] = '\033';
 	code[1] = '[';
 
-	ret = longcat(code, &length, y, 30, 0);
-	/* Make sure theres atleast room for ';' and `some' number ;) */
-	if (ret < 0 || length > 30) return NULL;
+	if (ulongcat(code, &length, y, CURSOR_NUM_LEN, 0) < 0)
+		return screen;
 
 	code[length++] = ';';
 
-	ret = longcat(code, &length, x, sizeof(code) - length, 0);
-	if (ret < 0 || length > 31) return NULL;
+	if (ulongcat(code, &length, x, CURSOR_NUM_LEN, 0) < 0)
+		return screen;
 
 	code[length++] = 'H';
 
 	return add_bytes_to_string(screen, code, length);
+#undef CURSOR_NUM_LEN
 }
 
 struct screen_state {
@@ -348,15 +343,22 @@ struct screen_state {
 	unsigned char underline;
 	unsigned char bold;
 	unsigned char attr;
+	/* Following should match struct screen_char color field. */
+#if defined(CONFIG_88_COLORS) || defined(CONFIG_256_COLORS)
 	unsigned char color[2];
+#else
+	unsigned char color[1];
+#endif
 };
 
-#ifdef CONFIG_256_COLORS
-#define compare_color(a, b)	(!memcmp((a), (b), 2))
-#define copy_color(a, b)	memcpy((a), (b), 2)
+#if defined(CONFIG_88_COLORS) || defined(CONFIG_256_COLORS)
+#define compare_color(a, b)	((a)[0] == (b)[0] && (a)[1] == (b)[1])
+#define copy_color(a, b)	do { (a)[0] = (b)[0]; (a)[1] = (b)[1]; } while (0)
+#define INIT_SCREEN_STATE 	{ 0xFF, 0xFF, 0xFF, 0, { 0xFF, 0xFF } }
 #else
 #define compare_color(a, b)	((a)[0] == (b)[0])
-#define copy_color(a, b)	((a)[0] = (b)[0])
+#define copy_color(a, b)	do { (a)[0] = (b)[0]; } while (0)
+#define INIT_SCREEN_STATE 	{ 0xFF, 0xFF, 0xFF, 0, { 0xFF } }
 #endif
 
 #define compare_bg_color(a, b)	(TERM_COLOR_BACKGROUND(a) == TERM_COLOR_BACKGROUND(b))
@@ -454,7 +456,7 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	add_char_data(screen, driver, ch->data, border);
 }
 
-#ifdef CONFIG_256_COLORS
+#if defined(CONFIG_88_COLORS) || defined(CONFIG_256_COLORS)
 static struct string color256_seqs[] = {
 	/* foreground: */	TERM_STRING("\033[0;38;5;%dm"),
 	/* background: */	TERM_STRING("\033[48;5;%dm"),
@@ -627,7 +629,7 @@ redraw_screen(struct terminal *term)
 {
 	struct screen_driver *driver;
 	struct string image;
-	struct screen_state state = { 0xFF, 0xFF, 0xFF, 0, { 0xFF, 0xFF } };
+	struct screen_state state = INIT_SCREEN_STATE;
 	struct terminal_screen *screen = term->screen;
 
 	if (!screen || screen->dirty_from > screen->dirty_to) return;
@@ -643,6 +645,11 @@ redraw_screen(struct terminal *term)
 	case COLOR_MODE_16:
 		add_chars(&image, term, driver, &state, add_char16);
 		break;
+#ifdef CONFIG_88_COLORS
+	case COLOR_MODE_88:
+		add_chars(&image, term, driver, &state, add_char256);
+		break;
+#endif
 #ifdef CONFIG_256_COLORS
 	case COLOR_MODE_256:
 		add_chars(&image, term, driver, &state, add_char256);
